@@ -1,6 +1,6 @@
-;;; gnus-uu.el --- extract (uu)encoded files in Gnus
+;;; gnus-uu.el --- extract (uu)encoded files in Gnus  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1985-1987, 1993-1998, 2000-2020 Free Software
+;; Copyright (C) 1985-1987, 1993-1998, 2000-2024 Free Software
 ;; Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
@@ -162,7 +162,7 @@ Note that this variable can be used in conjunction with the
 		 (regexp :format "%v")))
 
 (defcustom gnus-uu-ignore-files-by-type nil
-  "A regular expression saying what files that shouldn't be viewed, based on MIME file type.
+  "Regexp matching files that shouldn't be viewed, based on MIME file type.
 If, for instance, you want gnus-uu to ignore all audio files and all mpegs,
 you could say something like
 
@@ -224,7 +224,7 @@ Default is \"/tmp/\"."
   :type 'directory)
 
 (defcustom gnus-uu-do-not-unpack-archives nil
-  "Non-nil means that gnus-uu won't peek inside archives looking for files to display.
+  "If non-nil, gnus-uu won't peek inside archives looking for files to display.
 Default is nil."
   :group 'gnus-extract-archive
   :type 'boolean)
@@ -260,24 +260,25 @@ Default is t."
   "Non-nil means that files will be viewed with metamail.
 The gnus-uu viewing functions will be ignored and gnus-uu will try
 to guess at a content-type based on file name suffixes.  Default
-it nil."
+is nil."
   :group 'gnus-extract
   :type 'boolean)
+(make-obsolete-variable 'gnus-uu-view-with-metamail "don't use it." "29.1")
 
 (defcustom gnus-uu-unmark-articles-not-decoded nil
-  "Non-nil means that gnus-uu will mark articles that were unsuccessfully decoded as unread.
+  "If non-nil, gnus-uu will mark unsuccessfully decoded articles as unread.
 Default is nil."
   :group 'gnus-extract
   :type 'boolean)
 
 (defcustom gnus-uu-correct-stripped-uucode nil
-  "Non-nil means that gnus-uu will *try* to fix uuencoded files that have had trailing spaces deleted.
+  "If non-nil, *try* to fix uuencoded files that have had trailing spaces deleted.
 Default is nil."
   :group 'gnus-extract
   :type 'boolean)
 
 (defcustom gnus-uu-save-in-digest nil
-  "Non-nil means that gnus-uu, when asked to save without decoding, will save in digests.
+  "If non-nil, gnus-uu, when asked to save without decoding, will save in digests.
 If this variable is nil, gnus-uu will just save everything in a
 file without any embellishments.  The digesting almost conforms to RFC1153 -
 no easy way to specify any meaningful volume and issue numbers were found,
@@ -354,59 +355,63 @@ didn't work, and overwrite existing files.  Otherwise, ask each time."
 ;; Commands.
 
 (defun gnus-uu-decode-uu (&optional n)
-  "Uudecodes the current article."
-  (interactive "P")
-  (gnus-uu-decode-with-method 'gnus-uu-uustrip-article n))
+  "Uudecode the current article."
+  (interactive "P" gnus-article-mode gnus-summary-mode)
+  (gnus-uu-decode-with-method #'gnus-uu-uustrip-article n))
 
 (defun gnus-uu-decode-uu-and-save (n dir)
-  "Decodes and saves the resulting file."
+  "Decode and save the resulting file."
   (interactive
    (list current-prefix-arg
 	 (file-name-as-directory
 	  (read-directory-name "Uudecode and save in dir: "
-			  gnus-uu-default-dir
-			  gnus-uu-default-dir t))))
-  (gnus-uu-decode-with-method 'gnus-uu-uustrip-article n dir nil nil t))
+			       gnus-uu-default-dir
+			       gnus-uu-default-dir t)))
+   gnus-article-mode gnus-summary-mode)
+  (gnus-uu-decode-with-method #'gnus-uu-uustrip-article n dir nil nil t))
 
 (defun gnus-uu-decode-unshar (&optional n)
-  "Unshars the current article."
-  (interactive "P")
-  (gnus-uu-decode-with-method 'gnus-uu-unshar-article n nil nil 'scan t))
+  "Unshar the current article."
+  (interactive "P" gnus-article-mode gnus-summary-mode)
+  (gnus-uu-decode-with-method #'gnus-uu-unshar-article n nil nil 'scan t))
 
 (defun gnus-uu-decode-unshar-and-save (n dir)
-  "Unshars and saves the current article."
+  "Unshar and save the current article."
   (interactive
    (list current-prefix-arg
 	 (file-name-as-directory
 	  (read-directory-name "Unshar and save in dir: "
-			  gnus-uu-default-dir
-			  gnus-uu-default-dir t))))
-  (gnus-uu-decode-with-method 'gnus-uu-unshar-article n dir nil 'scan t))
+			       gnus-uu-default-dir
+			       gnus-uu-default-dir t)))
+   gnus-article-mode gnus-summary-mode)
+  (gnus-uu-decode-with-method #'gnus-uu-unshar-article n dir nil 'scan t))
 
 (defun gnus-uu-decode-save (n file)
-  "Saves the current article."
+  "Save the current article."
   (interactive
    (list current-prefix-arg
 	 (if gnus-uu-save-separate-articles
 	     (read-directory-name
 	      "Save articles in dir: " gnus-uu-default-dir gnus-uu-default-dir)
 	   (read-file-name
-	    "Save article in file: " gnus-uu-default-dir gnus-uu-default-dir))))
+	    "Save article in file: " gnus-uu-default-dir gnus-uu-default-dir)))
+   gnus-article-mode gnus-summary-mode)
   (setq gnus-uu-saved-article-name file)
-  (gnus-uu-decode-with-method 'gnus-uu-save-article n nil t))
+  (gnus-uu-decode-with-method #'gnus-uu-save-article n nil t))
 
 (defun gnus-uu-decode-binhex (n dir)
-  "Unbinhexes the current article."
+  "Unbinhex the current article."
   (interactive
    (list current-prefix-arg
 	 (file-name-as-directory
 	  (read-directory-name "Unbinhex and save in dir: "
-			  gnus-uu-default-dir
-			  gnus-uu-default-dir))))
+			       gnus-uu-default-dir
+			       gnus-uu-default-dir)))
+   gnus-article-mode gnus-summary-mode)
   (gnus-uu-initialize)
   (setq gnus-uu-binhex-article-name
 	(make-temp-file (expand-file-name "binhex" gnus-uu-work-dir)))
-  (gnus-uu-decode-with-method 'gnus-uu-binhex-article n dir))
+  (gnus-uu-decode-with-method #'gnus-uu-binhex-article n dir))
 
 (defun gnus-uu-decode-yenc (n dir)
   "Decode the yEnc-encoded current article."
@@ -414,61 +419,66 @@ didn't work, and overwrite existing files.  Otherwise, ask each time."
    (list current-prefix-arg
 	 (file-name-as-directory
 	  (read-directory-name "yEnc decode and save in dir: "
-			  gnus-uu-default-dir
-			  gnus-uu-default-dir))))
+			       gnus-uu-default-dir
+			       gnus-uu-default-dir)))
+   gnus-article-mode gnus-summary-mode)
   (setq gnus-uu-yenc-article-name nil)
-  (gnus-uu-decode-with-method 'gnus-uu-yenc-article n dir nil t))
+  (gnus-uu-decode-with-method #'gnus-uu-yenc-article n dir nil t))
 
 (defun gnus-uu-decode-uu-view (&optional n)
-  "Uudecodes and views the current article."
-  (interactive "P")
+  "Uudecode and view the current article."
+  (interactive "P" gnus-article-mode gnus-summary-mode)
   (let ((gnus-view-pseudos (or gnus-view-pseudos 'automatic)))
     (gnus-uu-decode-uu n)))
 
 (defun gnus-uu-decode-uu-and-save-view (n dir)
-  "Decodes, views and saves the resulting file."
+  "Decode, view and save the resulting file."
   (interactive
    (list current-prefix-arg
 	 (read-file-name "Uudecode, view and save in dir: "
 			 gnus-uu-default-dir
-			 gnus-uu-default-dir t)))
+			 gnus-uu-default-dir t))
+   gnus-article-mode gnus-summary-mode)
   (let ((gnus-view-pseudos (or gnus-view-pseudos 'automatic)))
     (gnus-uu-decode-uu-and-save n dir)))
 
 (defun gnus-uu-decode-unshar-view (&optional n)
-  "Unshars and views the current article."
-  (interactive "P")
+  "Unshar and view the current article."
+  (interactive "P" gnus-article-mode gnus-summary-mode)
   (let ((gnus-view-pseudos (or gnus-view-pseudos 'automatic)))
     (gnus-uu-decode-unshar n)))
 
 (defun gnus-uu-decode-unshar-and-save-view (n dir)
-  "Unshars and saves the current article."
+  "Unshar and save the current article."
   (interactive
    (list current-prefix-arg
 	 (read-file-name "Unshar, view and save in dir: "
 			 gnus-uu-default-dir
-			 gnus-uu-default-dir t)))
+			 gnus-uu-default-dir t))
+   gnus-article-mode gnus-summary-mode)
   (let ((gnus-view-pseudos (or gnus-view-pseudos 'automatic)))
     (gnus-uu-decode-unshar-and-save n dir)))
 
 (defun gnus-uu-decode-save-view (n file)
-  "Saves and views the current article."
+  "Save and view the current article."
   (interactive
    (list current-prefix-arg
 	 (if gnus-uu-save-separate-articles
 	     (read-directory-name "Save articles in dir: "
 				  gnus-uu-default-dir gnus-uu-default-dir)
 	   (read-file-name "Save articles in file: "
-			   gnus-uu-default-dir gnus-uu-default-dir))))
+			   gnus-uu-default-dir gnus-uu-default-dir)))
+   gnus-article-mode gnus-summary-mode)
   (let ((gnus-view-pseudos (or gnus-view-pseudos 'automatic)))
     (gnus-uu-decode-save n file)))
 
 (defun gnus-uu-decode-binhex-view (n file)
-  "Unbinhexes and views the current article."
+  "Unbinhex and view the current article."
   (interactive
    (list current-prefix-arg
 	 (read-file-name "Unbinhex, view and save in dir: "
-			 gnus-uu-default-dir gnus-uu-default-dir)))
+			 gnus-uu-default-dir gnus-uu-default-dir))
+   gnus-article-mode gnus-summary-mode)
   (gnus-uu-initialize)
   (setq gnus-uu-binhex-article-name
 	(make-temp-file (expand-file-name "binhex" gnus-uu-work-dir)))
@@ -479,8 +489,8 @@ didn't work, and overwrite existing files.  Otherwise, ask each time."
 ;; Digest and forward articles
 
 (defun gnus-uu-digest-mail-forward (&optional n post)
-  "Digests and forwards all articles in this series."
-  (interactive "P")
+  "Digest and forward all articles in this series."
+  (interactive "P" gnus-article-mode gnus-summary-mode)
   (gnus-uu-initialize)
   (let ((gnus-uu-save-in-digest t)
 	(file (make-temp-file (nnheader-concat gnus-uu-work-dir "forward")))
@@ -534,11 +544,11 @@ didn't work, and overwrite existing files.  Otherwise, ask each time."
 		    "Various"))))
 	(goto-char (point-min))
 	(when (re-search-forward "^Subject: ")
-	  (delete-region (point) (point-at-eol))
+          (delete-region (point) (line-end-position))
 	  (insert subject))
 	(goto-char (point-min))
 	(when (re-search-forward "^From:")
-	  (delete-region (point) (point-at-eol))
+          (delete-region (point) (line-end-position))
 	  (insert " " from))
 	(let ((message-forward-decoded-p t))
 	  (message-forward post t))))
@@ -546,7 +556,7 @@ didn't work, and overwrite existing files.  Otherwise, ask each time."
 
 (defun gnus-uu-digest-post-forward (&optional n)
   "Digest and forward to a newsgroup."
-  (interactive "P")
+  (interactive "P" gnus-article-mode gnus-summary-mode)
   (gnus-uu-digest-mail-forward n t))
 
 ;; Process marking.
@@ -569,14 +579,14 @@ didn't work, and overwrite existing files.  Otherwise, ask each time."
 
 (defun gnus-new-processable (unmarkp articles)
   (if unmarkp
-      (gnus-intersection gnus-newsgroup-processable articles)
+      (nreverse (seq-intersection gnus-newsgroup-processable articles #'eq))
     (gnus-set-difference articles gnus-newsgroup-processable)))
 
 (defun gnus-uu-mark-by-regexp (regexp &optional unmark)
   "Set the process mark on articles whose subjects match REGEXP.
 When called interactively, prompt for REGEXP.
 Optional UNMARK non-nil means unmark instead of mark."
-  (interactive "sMark (regexp): \nP")
+  (interactive "sMark (regexp): \nP" gnus-article-mode gnus-summary-mode)
   (save-excursion
     (let* ((articles (gnus-uu-find-articles-matching regexp))
 	   (new-marked (gnus-new-processable unmark articles)))
@@ -590,12 +600,12 @@ Optional UNMARK non-nil means unmark instead of mark."
 (defun gnus-uu-unmark-by-regexp (regexp)
   "Remove the process mark from articles whose subjects match REGEXP.
 When called interactively, prompt for REGEXP."
-  (interactive "sUnmark (regexp): ")
+  (interactive "sUnmark (regexp): " gnus-article-mode gnus-summary-mode)
   (gnus-uu-mark-by-regexp regexp t))
 
 (defun gnus-uu-mark-series (&optional silent)
   "Mark the current series with the process mark."
-  (interactive)
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (let* ((articles (gnus-uu-find-articles-matching))
 	 (l (length articles)))
     (while articles
@@ -608,7 +618,7 @@ When called interactively, prompt for REGEXP."
 
 (defun gnus-uu-mark-region (beg end &optional unmark)
   "Set the process mark on all articles between point and mark."
-  (interactive "r")
+  (interactive "r" gnus-article-mode gnus-summary-mode)
   (save-excursion
     (goto-char beg)
     (while (< (point) end)
@@ -620,22 +630,22 @@ When called interactively, prompt for REGEXP."
 
 (defun gnus-uu-unmark-region (beg end)
   "Remove the process mark from all articles between point and mark."
-  (interactive "r")
+  (interactive "r" gnus-article-mode gnus-summary-mode)
   (gnus-uu-mark-region beg end t))
 
 (defun gnus-uu-mark-buffer ()
   "Set the process mark on all articles in the buffer."
-  (interactive)
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (gnus-uu-mark-region (point-min) (point-max)))
 
 (defun gnus-uu-unmark-buffer ()
   "Remove the process mark on all articles in the buffer."
-  (interactive)
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (gnus-uu-mark-region (point-min) (point-max) t))
 
 (defun gnus-uu-mark-thread ()
-  "Marks all articles downwards in this thread."
-  (interactive)
+  "Mark all articles downwards in this thread."
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (gnus-save-hidden-threads
     (let ((level (gnus-summary-thread-level)))
       (while (and (gnus-summary-set-process-mark (gnus-summary-article-number))
@@ -645,8 +655,8 @@ When called interactively, prompt for REGEXP."
   (gnus-summary-position-point))
 
 (defun gnus-uu-unmark-thread ()
-  "Unmarks all articles downwards in this thread."
-  (interactive)
+  "Unmark all articles downwards in this thread."
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (let ((level (gnus-summary-thread-level)))
     (while (and (gnus-summary-remove-process-mark
 		 (gnus-summary-article-number))
@@ -656,7 +666,7 @@ When called interactively, prompt for REGEXP."
 
 (defun gnus-uu-invert-processable ()
   "Invert the list of process-marked articles."
-  (interactive)
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (let ((data gnus-newsgroup-data)
 	number)
     (save-excursion
@@ -669,7 +679,7 @@ When called interactively, prompt for REGEXP."
 
 (defun gnus-uu-mark-over (&optional score)
   "Mark all articles with a score over SCORE (the prefix)."
-  (interactive "P")
+  (interactive "P" gnus-article-mode gnus-summary-mode)
   (let ((score (or score gnus-summary-default-score 0))
 	(data gnus-newsgroup-data))
     (save-excursion
@@ -684,7 +694,7 @@ When called interactively, prompt for REGEXP."
 
 (defun gnus-uu-mark-sparse ()
   "Mark all series that have some articles marked."
-  (interactive)
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (let ((marked (nreverse gnus-newsgroup-processable))
 	subject articles total headers)
     (unless marked
@@ -708,7 +718,7 @@ When called interactively, prompt for REGEXP."
 
 (defun gnus-uu-mark-all ()
   "Mark all articles in \"series\" order."
-  (interactive)
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (setq gnus-newsgroup-processable nil)
   (save-excursion
     (let ((data gnus-newsgroup-data)
@@ -728,33 +738,33 @@ When called interactively, prompt for REGEXP."
 
 (defun gnus-uu-decode-postscript (&optional n)
   "Gets PostScript of the current article."
-  (interactive "P")
-  (gnus-uu-decode-with-method 'gnus-uu-decode-postscript-article n))
+  (interactive "P" gnus-article-mode gnus-summary-mode)
+  (gnus-uu-decode-with-method #'gnus-uu-decode-postscript-article n))
 
 (defun gnus-uu-decode-postscript-view (&optional n)
   "Gets and views the current article."
-  (interactive "P")
+  (interactive "P" gnus-article-mode gnus-summary-mode)
   (let ((gnus-view-pseudos (or gnus-view-pseudos 'automatic)))
     (gnus-uu-decode-postscript n)))
 
 (defun gnus-uu-decode-postscript-and-save (n dir)
-  "Extracts PostScript and saves the current article."
-  (interactive
-   (list current-prefix-arg
-	 (file-name-as-directory
-	  (read-directory-name "Save in dir: "
-			  gnus-uu-default-dir
-			  gnus-uu-default-dir t))))
-  (gnus-uu-decode-with-method 'gnus-uu-decode-postscript-article
+  "Extract PostScript and save the current article."
+  (interactive (list current-prefix-arg
+		     (file-name-as-directory
+		      (read-directory-name "Save in dir: "
+					   gnus-uu-default-dir
+					   gnus-uu-default-dir t)))
+	       gnus-article-mode gnus-summary-mode)
+  (gnus-uu-decode-with-method #'gnus-uu-decode-postscript-article
 			      n dir nil nil t))
 
 (defun gnus-uu-decode-postscript-and-save-view (n dir)
-  "Decodes, views and saves the resulting file."
-  (interactive
-   (list current-prefix-arg
-	 (read-file-name "Where do you want to save the file(s)? "
-			 gnus-uu-default-dir
-			 gnus-uu-default-dir t)))
+  "Decode, view and save the resulting file."
+  (interactive (list current-prefix-arg
+		     (read-file-name "Where do you want to save the file(s)? "
+				     gnus-uu-default-dir
+				     gnus-uu-default-dir t))
+	       gnus-article-mode gnus-summary-mode)
   (let ((gnus-view-pseudos (or gnus-view-pseudos 'automatic)))
     (gnus-uu-decode-postscript-and-save n dir)))
 
@@ -977,7 +987,7 @@ When called interactively, prompt for REGEXP."
 (defvar gnus-uu-binhex-end-line
   ":$")
 
-(defun gnus-uu-binhex-article (buffer in-state)
+(defun gnus-uu-binhex-article (buffer _in-state)
   (let (state start-char)
     (with-current-buffer buffer
       (widen)
@@ -1014,11 +1024,11 @@ When called interactively, prompt for REGEXP."
 
 ;; yEnc
 
-(defun gnus-uu-yenc-article (buffer in-state)
+(defun gnus-uu-yenc-article (_buffer _in-state)
   (with-current-buffer gnus-original-article-buffer
     (widen)
     (let ((file-name (yenc-extract-filename))
-	  state start-char)
+	  state) ;; start-char
       (when (not file-name)
 	(setq state (list 'wrong-type)))
 
@@ -1046,7 +1056,7 @@ When called interactively, prompt for REGEXP."
 
 ;; PostScript
 
-(defun gnus-uu-decode-postscript-article (process-buffer in-state)
+(defun gnus-uu-decode-postscript-article (process-buffer _in-state)
   (let ((state (list 'ok))
 	start-char end-char file-name)
     (with-current-buffer process-buffer
@@ -1196,11 +1206,11 @@ When called interactively, prompt for REGEXP."
 
 	;; Expand numbers, sort, and return the list of article
 	;; numbers.
-	(mapcar 'cdr
+	(mapcar #'cdr
 		(sort (gnus-uu-expand-numbers
 		       list-of-subjects
 		       (not do-not-translate))
-		      'gnus-uu-string<))))))
+		      #'gnus-uu-string<))))))
 
 (defun gnus-uu-expand-numbers (string-list &optional translate)
   ;; Takes a list of strings and "expands" all numbers in all the
@@ -1278,13 +1288,15 @@ When called interactively, prompt for REGEXP."
       (when dont-unmark-last-article
 	(setq gnus-uu-has-been-grabbed (list art))))))
 
+(defvar gnus-asynchronous)
+
 ;; This function takes a list of articles and a function to apply to
 ;; each article grabbed.
 ;;
 ;; This function returns a list of files decoded if the grabbing and
 ;; the process-function has been successful and nil otherwise.
 (defun gnus-uu-grab-articles (articles process-function
-				       &optional sloppy limit no-errors)
+				       &optional sloppy limit _no-errors)
   (require 'gnus-async)
   (let ((state 'first)
 	(gnus-asynchronous nil)
@@ -1423,7 +1435,7 @@ When called interactively, prompt for REGEXP."
   "View FILE using the gnus-uu methods."
   (let ((action (gnus-uu-get-action file)))
     (gnus-execute-command
-     (if (string-match "%" action)
+     (if (string-search "%" action)
 	 (format action file)
        (concat action " " file))
      (eq gnus-view-pseudos 'not-confirm))))
@@ -1452,10 +1464,10 @@ When called interactively, prompt for REGEXP."
 	  (setq subject (substring subject (match-end 0)))))
     (or part "")))
 
-(defun gnus-uu-uudecode-sentinel (process event)
+(defun gnus-uu-uudecode-sentinel (process _event)
   (delete-process (get-process process)))
 
-(defun gnus-uu-uustrip-article (process-buffer in-state)
+(defun gnus-uu-uustrip-article (process-buffer _in-state)
   ;; Uudecodes a file asynchronously.
   (with-current-buffer process-buffer
     (let ((state (list 'wrong-type))
@@ -1576,7 +1588,7 @@ Gnus might fail to display all of it.")
 
 ;; This function is used by `gnus-uu-grab-articles' to treat
 ;; a shared article.
-(defun gnus-uu-unshar-article (process-buffer in-state)
+(defun gnus-uu-unshar-article (process-buffer _in-state)
   (let ((state (list 'ok))
 	start-char)
     (with-current-buffer process-buffer
@@ -1587,8 +1599,7 @@ Gnus might fail to display all of it.")
 	  (save-excursion
 	    (switch-to-buffer (current-buffer))
 	    (delete-other-windows)
-	    (let ((buffer (get-buffer-create (generate-new-buffer-name
-					      "*Warning*"))))
+            (let ((buffer (generate-new-buffer "*Warning*")))
 	      (unless
 		  (unwind-protect
 		      (with-current-buffer buffer
@@ -1596,7 +1607,7 @@ Gnus might fail to display all of it.")
 				 gnus-uu-unshar-warning))
 			(goto-char (point-min))
 			(display-buffer buffer)
-			(yes-or-no-p "This is a shell archive, unshar it? "))
+                        (yes-or-no-p "This is a shell archive, unshar it?"))
 		    (kill-buffer buffer))
 		(setq state (list 'error))))))
 	(unless (memq 'error state)
@@ -1611,7 +1622,7 @@ Gnus might fail to display all of it.")
     state))
 
 ;; `gnus-uu-choose-action' chooses what action to perform given the name
-;; and `gnus-uu-file-action-list'.  Returns either nil if no action is
+;; and FILE-ACTION-LIST.  Return either nil if no action is
 ;; found, or the name of the command to run if such a rule is found.
 (defun gnus-uu-choose-action (file-name file-action-list &optional no-ignore)
   (let ((action-list (copy-sequence file-action-list))
@@ -1674,7 +1685,7 @@ Gnus might fail to display all of it.")
     did-unpack))
 
 (defun gnus-uu-dir-files (dir)
-  (let ((dirs (directory-files dir t (rx (or (not ".") "..."))))
+  (let ((dirs (directory-files dir t directory-files-no-dot-files-regexp))
 	files file)
     (while dirs
       (if (file-directory-p (setq file (car dirs)))
@@ -1752,7 +1763,7 @@ Gnus might fail to display all of it.")
 	    (unless (looking-at (concat gnus-uu-begin-string "\\|"
 					gnus-uu-end-string))
 	      (when (not found)
-		(setq length (- (point-at-eol) (point-at-bol))))
+                (setq length (- (line-end-position) (line-beginning-position))))
 	      (setq found t)
 	      (beginning-of-line)
 	      (setq beg (point))
@@ -1781,8 +1792,8 @@ Gnus might fail to display all of it.")
 		 gnus-uu-tmp-dir)))
 
       (setq gnus-uu-work-dir
-	    (make-temp-file (concat gnus-uu-tmp-dir "gnus") 'dir))
-      (gnus-set-file-modes gnus-uu-work-dir 448)
+	    (with-file-modes #o700
+	      (make-temp-file (concat gnus-uu-tmp-dir "gnus") 'dir)))
       (setq gnus-uu-work-dir (file-name-as-directory gnus-uu-work-dir))
       (push (cons gnus-newsgroup-name gnus-uu-work-dir)
 	    gnus-uu-tmp-alist))))
@@ -1831,8 +1842,8 @@ Gnus might fail to display all of it.")
 
 ;; Initializing
 
-(add-hook 'gnus-summary-prepare-exit-hook 'gnus-uu-clean-up)
-(add-hook 'gnus-summary-prepare-exit-hook 'gnus-uu-delete-work-dir)
+(add-hook 'gnus-summary-prepare-exit-hook #'gnus-uu-clean-up)
+(add-hook 'gnus-summary-prepare-exit-hook #'gnus-uu-delete-work-dir)
 
 
 
@@ -1858,7 +1869,7 @@ uuencode and adds MIME headers."
 		(function :tag "Other")))
 
 (defcustom gnus-uu-post-include-before-composing nil
-  "Non-nil means that gnus-uu will ask for a file to encode before you compose the article.
+  "If non-nil, gnus-uu asks for a file to encode before you compose the article.
 If this variable is t, you can either include an encoded file with
 \\[gnus-uu-post-insert-binary-in-article] or have one included for you when you post the article."
   :group 'gnus-extract-post
@@ -1915,7 +1926,7 @@ is t."
 			  (gnus-uu-post-insert-binary)))))
 
 (defun gnus-uu-post-insert-binary-in-article ()
-  "Inserts an encoded file in the buffer.
+  "Insert an encoded file in the buffer.
 The user will be asked for a file name."
   (interactive)
   (save-excursion
@@ -1950,6 +1961,7 @@ The user will be asked for a file name."
 		  (gnus-uu-choose-action file-name gnus-uu-ext-to-mime-list)
 		  file-name))
   (insert (format "Content-Transfer-Encoding: %s\n\n" encoding))
+  ;; FIXME: Shouldn't we set-buffer before saving the restriction?  --Stef
   (save-restriction
     (set-buffer gnus-message-buffer)
     (goto-char (point-min))
@@ -2056,7 +2068,7 @@ If no file has been included, the user will be asked for a file."
     (goto-char (point-min))
     (re-search-forward
      (concat "^" (regexp-quote mail-header-separator) "$") nil t)
-    (setq header (buffer-substring (point-min) (point-at-bol)))
+    (setq header (buffer-substring (point-min) (line-beginning-position)))
 
     (goto-char (point-min))
     (when gnus-uu-post-separate-description
