@@ -1,8 +1,9 @@
-;;; url-gw.el --- Gateway munging for URL loading
+;;; url-gw.el --- Gateway munging for URL loading  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1997-1998, 2004-2012  Free Software Foundation, Inc.
+;; Copyright (C) 1997-1998, 2004-2024 Free Software Foundation, Inc.
 
 ;; Author: Bill Perry <wmperry@gnu.org>
+;; Maintainer: emacs-devel@gnu.org
 ;; Keywords: comm, data, processes
 
 ;; This file is part of GNU Emacs.
@@ -18,14 +19,14 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
+
+;;; Commentary:
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
 (require 'url-vars)
-
-;; Fixme: support SSH explicitly or via a url-gateway-rlogin-program?
+(require 'url-parse)
 
 (autoload 'socks-open-network-stream "socks")
 
@@ -39,7 +40,7 @@
   :group 'url-gateway)
 
 (defcustom url-gateway-prompt-pattern
-  "^[^#$%>;]*[#$%>;] *" ;; "bash\\|\$ *\r?$\\|> *\r?"
+  "^[^#$%>;]*[#$%>;] *" ;; "bash\\|[$>] *\r?$"
   "A regular expression matching a shell prompt."
   :type 'regexp
   :group 'url-gateway)
@@ -48,17 +49,20 @@
   "What hostname to actually rlog into before doing a telnet."
   :type '(choice (const nil) string)
   :group 'url-gateway)
+(make-obsolete-variable 'url-gateway-rlogin-host nil "29.1")
 
 (defcustom url-gateway-rlogin-user-name nil
   "Username to log into the remote machine with when using rlogin."
   :type '(choice (const nil) string)
   :group 'url-gateway)
+(make-obsolete-variable 'url-gateway-rlogin-user-name nil "29.1")
 
 (defcustom url-gateway-rlogin-parameters '("telnet" "-8")
   "Parameters to `url-open-rlogin'.
 This list will be used as the parameter list given to rsh."
   :type '(repeat string)
   :group 'url-gateway)
+(make-obsolete-variable 'url-gateway-rlogin-parameters nil "29.1")
 
 (defcustom url-gateway-telnet-host nil
   "What hostname to actually login to before doing a telnet."
@@ -138,6 +142,7 @@ linked Emacs under SunOS 4.x."
 ;; Stolen from red gnus nntp.el
 (defun url-open-rlogin (name buffer host service)
   "Open a connection using rsh."
+  (declare (obsolete nil "29.1"))
   (if (not (stringp service))
       (setq service (int-to-string service)))
   (let ((proc (if url-gateway-rlogin-user-name
@@ -190,7 +195,7 @@ linked Emacs under SunOS 4.x."
 	 proc (concat (mapconcat 'identity
 				 (append url-gateway-telnet-parameters
 					 (list host service)) " ") "\n"))
-	(url-wait-for-string "^\r*Escape character.*\r*\n+" proc)
+	(url-wait-for-string "^\r*Escape character.*\n+" proc)
 	(delete-region (point-min) (match-end 0))
 	(process-send-string proc "\^]\n")
 	(url-wait-for-string "^telnet" proc)
@@ -202,54 +207,65 @@ linked Emacs under SunOS 4.x."
 	(delete-region (point) (point-max)))
       proc)))
 
+(defvar url-gw-rlogin-obsolete-warned-once nil)
+(make-obsolete-variable 'url-gw-rlogin-obsolete-warned-once nil "29.1")
+
 ;;;###autoload
-(defun url-open-stream (name buffer host service)
+(defun url-open-stream (name buffer host service &optional gateway-method)
   "Open a stream to HOST, possibly via a gateway.
 Args per `open-network-stream'.
 Will not make a connection if `url-gateway-unplugged' is non-nil.
-Might do a non-blocking connection; use `process-status' to check."
+Might do a non-blocking connection; use `process-status' to check.
+
+Optional arg GATEWAY-METHOD specifies the gateway to be used,
+overriding the value of `url-gateway-method'."
   (unless url-gateway-unplugged
-    (let ((gw-method (if (and url-gateway-local-host-regexp
-			      (not (eq 'tls url-gateway-method))
-			      (not (eq 'ssl url-gateway-method))
-			      (string-match
-			       url-gateway-local-host-regexp
-			       host))
-			 'native
-		       url-gateway-method))
-	  ;; An attempt to deal with denied connections, and attempt
-	  ;; to reconnect
-	  (cur-retries 0)
-	  (retry t)
-	  (errobj nil)
-	  (conn nil))
+    (let* ((gwm (or gateway-method url-gateway-method))
+           (gw-method (if (and url-gateway-local-host-regexp
+                               (not (eq 'tls gwm))
+                               (not (eq 'ssl gwm))
+                               (string-match
+                                url-gateway-local-host-regexp
+                                host))
+                          'native
+                        gwm))
+	   ;; An attempt to deal with denied connections, and attempt
+	   ;; to reconnect
+	   ;; (cur-retries 0)
+	   ;; (retry t)
+	   (conn nil))
 
       ;; If the user told us to do DNS for them, do it.
       (if url-gateway-broken-resolution
 	  (setq host (url-gateway-nslookup-host host)))
 
-      (condition-case errobj
+      (condition-case nil
 	  ;; This is a clean way to ensure the new process inherits the
 	  ;; right coding systems in both Emacs and XEmacs.
 	  (let ((coding-system-for-read 'binary)
 		(coding-system-for-write 'binary))
-	    (setq conn (case gw-method
-			 ((tls ssl native)
+	    (setq conn (pcase gw-method
+			 ((or 'tls 'ssl 'native)
 			  (if (eq gw-method 'native)
 			      (setq gw-method 'plain))
 			  (open-network-stream
 			   name buffer host service
 			   :type gw-method
 			   ;; Use non-blocking socket if we can.
-			   :nowait (featurep 'make-network-process
-					     '(:nowait t))))
-			 (socks
+			   :nowait (and (featurep 'make-network-process)
+                                        (url-asynchronous url-current-object)
+                                        '(:nowait t))))
+                         ('socks
 			  (socks-open-network-stream name buffer host service))
-			 (telnet
+			 ('telnet
 			  (url-open-telnet name buffer host service))
-			 (rlogin
-			  (url-open-rlogin name buffer host service))
-			 (otherwise
+			 ('rlogin
+                          (unless url-gw-rlogin-obsolete-warned-once
+                            (lwarn 'url :error "Setting `url-gateway-method' to `rlogin' is obsolete")
+                            (setq url-gw-rlogin-obsolete-warned-once t))
+                          (with-suppressed-warnings ((obsolete url-open-rlogin))
+                            (url-open-rlogin name buffer host service)))
+			 (_
 			  (error "Bad setting of url-gateway-method: %s"
 				 url-gateway-method))))))
       conn)))
